==> nodehandler-utils-4.1.0.tgz: package/src/http-properties.js <==
const contentTypeParser = require('content-type');

/**
 * JSON schema for Advanced HTTP Properties rendered in Admin UI properties panel
 */
const advancedProperties = {
	'Content-Type': {
		title: 'Content-Type',
		description: 'This is the expected body format that you send.',
		type: 'string',
		group: 'Advanced HTTP Options'
	},
	Accept: {
		title: 'Accept',
		description: 'This is the response format you expect to receive.',
		type: 'string',
		group: 'Advanced HTTP Options'
	},
	'Decode Response': {
		title: 'Decode Response',
		description: 'Controls how the response is decoded. If auto, a best-effort attempt will be made to decode the response according to its content-type header. For example, If the content-type is "application/json", the response will decoded as JSON object, or if "text/plain" it will be returned as a string, but if it cannot decode, it will be returned as a NodeJS Buffer. If binary, no decoding is attempted and the response will be returned as a NodeJS Buffer.',
		type: 'string',
		enum: [ 'auto', 'binary' ],
		default: 'auto',
		group: 'Advanced HTTP Options'
	}
};

/**
 * Constructs and returns advanced HTTP properties that will be rendered as part of a method.
 * Some of them are based on Swagger `consumes` and `produces` per method or on global level.
 * Others are pure configuration properties.
 *
 * @param {object} swagger the swagger document.
 * @param {object} method the method.
 * @param {boolean} hasBodyParam if the method has body parameter.
 * @returns {object} object with properties categorized by their use e.g. header, config etc.
 */
module.exports = (swagger, method, hasBodyParam) => {
	const consumes = method.consumes || swagger.consumes || [];
	const produces = method.produces || swagger.produces || [];
	const consumesInfo = getMimeTypesInfo(consumes);
	const producesInfo = getMimeTypesInfo(produces);
	// The properties here are attached as advanced http properties to the method.
	// They are categorized according to their useage as requester metadata.
	const properties = {
		header: [],
		config: []
	};
	if (hasBodyParam) {
		// Content type relevant only if we have body parameter
		if (consumesInfo.exist) {
			properties.header.push({
				name: '_content-type',
				value: {
					...advancedProperties['Content-Type'],
					enum: consumesInfo.filtered,
					default: consumesInfo.defaultValue
				}
			});
		} else {
			// consumes not specified
			properties.header.push({
				name: '_content-type',
				value: {
					...advancedProperties['Content-Type'],
					// for backward compatibility set to json
					default: 'application/json'
				}
			});
		}
	}
	if (producesInfo.exist) {
		properties.header.push({
			name: '_accept',
			value: {
				...advancedProperties['Accept'],
				enum: producesInfo.filtered,
				default: producesInfo.defaultValue
			}
		});
	} else {
		// produces not specified
		properties.header.push({
			name: '_accept',
			value: {
				...advancedProperties['Accept'],
				// for backward compatibility set to json
				default: 'application/json'
			}
		});
	}

	/** We push it into config becase the value of this property is not send as part of the http request.
	 * It is used to configure the request itself. */
	properties.config.push({
		name: '_decode-response',
		value: advancedProperties['Decode Response']
	});

	return properties;
};

function filterInvalidTypes(mimeTypes) {
	return mimeTypes.filter((type) => {
		return parseContentType(type) !== undefined;
	});
}

/**
 * Calculates the default content type value.
 * Order of preference: application/json, application/XXX+json, banana.
 * Works on filtered collection of valid content types.
 *
 * @param {array} mimeTypes the list with the available content types.
 * @returns {object} with information about:
 *   exist: if mime type/types has been specified
 *   defaultValue: default value to use
 *   filtered: array with only valid mime types from all specified
 */
function getMimeTypesInfo(mimeTypes) {
	const filtered = filterInvalidTypes(mimeTypes);
	const mimeTypeInfo = {
		exist: false,
		defaultValue: undefined,
		filtered
	};
	if (filtered.length === 0) {
		return mimeTypeInfo;
	}
	if (filtered.length === 1) {
		return getFirst(filtered, mimeTypeInfo);
	}

	for (let i = 0; i < filtered.length; i++) {
		const parsed = parseContentType(filtered[i]);
		if (isJSON(parsed)) {
			// if we find exact match just set it and exit the cycle
			// this is with highest priority and we don't want to search more
			mimeTypeInfo.exist = true;
			mimeTypeInfo.defaultValue = parsed.type;
			break;
		} else if (isPlusJSON(parsed)) {
			// if we find json like type we set is as default
			// but we continue because we need to search for exact match
			mimeTypeInfo.exist = true;
			mimeTypeInfo.defaultValue = parsed.type;
		}
	}

	// Default type has not been found so return the first
	// If one of the types we dafult to has been found return it
	return !mimeTypeInfo.defaultValue ? getFirst(filtered, mimeTypeInfo) : mimeTypeInfo;
}

function getFirst(mimeTypes, mimeTypeInfo) {
	const parsed = parseContentType(mimeTypes[0]);
	mimeTypeInfo.exist = true;
	mimeTypeInfo.defaultValue = parsed.type;
	return mimeTypeInfo;
}

/**
 * Parse the content type with the help of content-type library.
 *
 * @param {string} contentType the content type
 * @returns {object|undefined} object with the information for the parsed content type or undefined in case of exception
 */
function parseContentType(contentType) {
	try {
		return contentTypeParser.parse(contentType);
	} catch (e) {
		// content-type library could fail on wrong types so returned undefined in that case
		// https://github.com/jshttp/content-type#contenttypeparsestring
		return undefined;
	}
}

/**
 * @param {object} parsed the parsed content type with https://www.npmjs.com/package/content-type
 * @returns {boolean} true if it is application/json
 */
function isJSON(parsed) {
	return parsed.type === 'application/json';
}

/**
 * @param {object} parsed the parsed content type with https://www.npmjs.com/package/content-type
 * @returns {boolean} true if it is of +JSON family
 */
function isPlusJSON(parsed) {
	return parsed.type.match(/\+?json$/) !== null;
}

==> nodehandler-utils-4.2.0.tgz: package/src/http-properties.js <==
const contentTypeParser = require('content-type');

/**
 * JSON schema for Advanced HTTP Properties rendered in Admin UI properties panel
 */
const advancedProperties = {
	'Content-Type': {
		title: 'Content-Type',
		description: 'This is the expected payload format that you send.',
		type: 'string',
		group: 'Advanced HTTP Options'
	},
	Accept: {
		title: 'Accept',
		description: 'This is the response format you expect to receive.',
		type: 'string',
		group: 'Advanced HTTP Options'
	},
	'Decode Response': {
		title: 'Decode Response',
		description: 'Controls how the response is decoded. If auto, a best-effort attempt will be made to decode the response according to its content-type header. For example, If the content-type is "application/json", the response will decoded as JSON object, or if "text/plain" it will be returned as a string, but if it cannot decode, it will be returned as a NodeJS Buffer. If binary, no decoding is attempted and the response will be returned as a NodeJS Buffer.',
		type: 'string',
		enum: [ 'auto', 'binary' ],
		default: 'auto',
		group: 'Advanced HTTP Options'
	}
};

/**
 * The possible payload types
 */
const payloadTypes = {
	body: 'body',
	formData: 'formData'
};

/**
 * Constructs and returns advanced HTTP properties that will be rendered as part of a method.
 * Some of them are based on Swagger `consumes` and `produces` per method or on global level.
 * Others are pure configuration properties.
 *
 * @param {object} swagger the swagger document.
 * @param {object} method the method.
 * @param {string} payloadType body, formData or undefined depending on how the payload is described.
 * @returns {object} object with properties categorized by their use e.g. header, config etc.
 */
module.exports = (swagger, method, payloadType) => {
	// The properties here are attached as advanced http properties to the method.
	// They are categorized according to their useage as requester metadata.
	const properties = {
		header: [],
		config: []
	};

	addConsumesInfo(swagger, method, payloadType, properties);
	addProducesInfo(swagger, method, properties);

	/** We push it into config becase the value of this property is not send as part of the http request.
	 * It is used to configure the request itself. */
	properties.config.push({
		name: '_decode-response',
		value: advancedProperties['Decode Response']
	});

	return properties;
};

function addConsumesInfo(swagger, method, payloadType, properties) {
	const consumes = method.consumes || swagger.consumes || [];
	const consumesInfo = getMimeTypesInfo(consumes);
	if (payloadType === payloadTypes.body) {
		// Content type relevant only if we have body parameter
		if (consumesInfo.exist) {
			properties.header.push({
				name: '_content-type',
				value: {
					...advancedProperties['Content-Type'],
					enum: consumesInfo.filtered,
					default: consumesInfo.defaultValue
				}
			});
		} else {
			// consumes not specified
			properties.header.push({
				name: '_content-type',
				value: {
					...advancedProperties['Content-Type'],
					// for backward compatibility set to json
					default: 'application/json'
				}
			});
		}
	} else if (payloadType === payloadTypes.formData) {
		const providedTypes = filterInvalidTypes(consumes);
		if (providedTypes.length > 0) {
			// consumes is specified within the Swagger
			properties.header.push({
				name: '_content-type',
				value: {
					...advancedProperties['Content-Type'],
					enum: providedTypes,
					default: providedTypes[0]
				}
			});
		} else {
			// consumes not specified so we render string
			properties.header.push({
				name: '_content-type',
				value: {
					...advancedProperties['Content-Type'],
					default: 'application/x-www-form-urlencoded'
				}
			});
		}
	}
}

function addProducesInfo(swagger, method, properties) {
	const produces = method.produces || swagger.produces || [];
	const producesInfo = getMimeTypesInfo(produces);
	if (producesInfo.exist) {
		properties.header.push({
			name: '_accept',
			value: {
				...advancedProperties['Accept'],
				enum: producesInfo.filtered,
				default: producesInfo.defaultValue
			}
		});
	} else {
		// produces not specified
		properties.header.push({
			name: '_accept',
			value: {
				...advancedProperties['Accept'],
				// for backward compatibility set to json
				default: 'application/json'
			}
		});
	}
}

function filterInvalidTypes(mimeTypes) {
	return mimeTypes.filter((type) => {
		return parseContentType(type) !== undefined;
	});
}

/**
 * Calculates the default content type value.
 * Order of preference: application/json, application/XXX+json, banana.
 * Works on filtered collection of valid content types.
 *
 * @param {array} mimeTypes the list with the available content types.
 * @returns {object} with information about:
 *   exist: if mime type/types has been specified
 *   defaultValue: default value to use
 *   filtered: array with only valid mime types from all specified
 */
function getMimeTypesInfo(mimeTypes) {
	const filtered = filterInvalidTypes(mimeTypes);
	const mimeTypeInfo = {
		exist: false,
		defaultValue: undefined,
		filtered
	};
	if (filtered.length === 0) {
		return mimeTypeInfo;
	}
	if (filtered.length === 1) {
		return getFirst(filtered, mimeTypeInfo);
	}

	for (let i = 0; i < filtered.length; i++) {
		const parsed = parseContentType(filtered[i]);
		if (isJSON(parsed)) {
			// if we find exact match just set it and exit the cycle
			// this is with highest priority and we don't want to search more
			mimeTypeInfo.exist = true;
			mimeTypeInfo.defaultValue = parsed.type;
			break;
		} else if (isPlusJSON(parsed)) {
			// if we find json like type we set is as default
			// but we continue because we need to search for exact match
			mimeTypeInfo.exist = true;
			mimeTypeInfo.defaultValue = parsed.type;
		}
	}

	// Default type has not been found so return the first
	// If one of the types we dafult to has been found return it
	return !mimeTypeInfo.defaultValue ? getFirst(filtered, mimeTypeInfo) : mimeTypeInfo;
}

function getFirst(mimeTypes, mimeTypeInfo) {
	const parsed = parseContentType(mimeTypes[0]);
	mimeTypeInfo.exist = true;
	mimeTypeInfo.defaultValue = parsed.type;
	return mimeTypeInfo;
}

/**
 * Parse the content type with the help of content-type library.
 *
 * @param {string} contentType the content type
 * @returns {object|undefined} object with the information for the parsed content type or undefined in case of exception
 */
function parseContentType(contentType) {
	try {
		return contentTypeParser.parse(contentType);
	} catch (e) {
		// content-type library could fail on wrong types so returned undefined in that case
		// https://github.com/jshttp/content-type#contenttypeparsestring
		return undefined;
	}
}

/**
 * @param {object} parsed the parsed content type with https://www.npmjs.com/package/content-type
 * @returns {boolean} true if it is application/json
 */
function isJSON(parsed) {
	return parsed.type === 'application/json';
}

/**
 * @param {object} parsed the parsed content type with https://www.npmjs.com/package/content-type
 * @returns {boolean} true if it is of +JSON family
 */
function isPlusJSON(parsed) {
	return parsed.type.match(/\+?json$/) !== null;
}
